# Chooser script for CD Shell -- Feb 2005 -- Mike Ter Louw

# Run the optimizer if it hasn't been run yet.
if (defined[c.prep.done])
	then goto setup

set c.prep.return = "$(c.path)/chooser.cds"
script $(c.path)/ch_prep.cds

#====================================================================
setup:
set c.escaped = 0
set c.finished = 0
set c.selected = 0
set _color = $textColor
set _cursor.x = $textCursorX
set _cursor.y = $textCursorY

# Configure mouse
if (!$c.mouse); then goto defineBG
set c.mouse = $mouse
if (!$c.mouse); then goto defineBG
set _m.vis = $mouseVisible
set mouseVisible = 1

# Background
defineBG:
if ($bg.custom); then goto defineMenu
set bg.char = " "
set bg.color = color[white on white]

# Menu window
defineMenu:
if ($c.menu.custom); then goto defineMsg
set c.menu.width = 40	# Max = 76, Min = Longest menu item + 4
set c.menu.height = 11	# Max = 13, Min = 5
set c.menu.win.color = color[white on brightblue]
set c.menu.shadow.color = color[grey on white]
set c.scroll.color = color[yellow on brightblue]
set c.menu.text.color = color[white on brightblue]
set c.menu.selected.color = color[white on blue]
set c.menu.clicked.color = color[white on red]

# Message window
defineMsg:
if ($c.msg.custom); then goto drawBG
set c.msg.win.color = color[white on brightgrey]
set c.msg.shadow.color = color[grey on white]
set c.msg.text.color = color[grey on brightgrey]

#====================================================================
# Draw the background.
drawBG:
set bg.start = 1
set bg.size = 23
call drawBackground

#================================================
# Message Window Definition
set win.color = $c.msg.win.color
set win.shadow.color = $c.msg.shadow.color
set win.width = 72
set win.height = 6
set win.position.x = 4
set win.position.y = 16
call drawWindow

#================================================
# Menu Window Definition
set menu.width = $c.menu.width
set menu.height = $c.menu.height
set menu.position.x = (80 - $menu.width) / 2
set menu.position.y = (15 - $menu.height) / 2 + 1
set menu.extent.x = $menu.position.x + $menu.width - 1
set menu.extent.y = $menu.position.y + $menu.height - 1

set win.color = $c.menu.win.color
set win.shadow.color = $c.menu.shadow.color
set win.width = $menu.width
set win.height = $menu.height
set win.position.x = $menu.position.x
set win.position.y = $menu.position.y
set scroll.color = $c.scroll.color
call drawWindow

#================================================
# Draw the list of items into the menu.
set menu.start = 0
set c.selected = 0
call updateSelected

#====================================================================
mainLoop:
if ($c.mouse)
	then mgetkey
	else getkey
if ($lastKey == key[mouse]); then goto mouseHandler
if ($lastKey == key[up]); then goto scrollUp
if ($lastKey == key[down]); then goto scrollDown
if ($lastKey == key[pgup]); then goto pageUp
if ($lastKey == key[pgdn]); then goto pageDown
if ($lastKey == key[enter]); then goto finish
if ($lastKey == key[esc]); then goto escape
goto mainLoop

#====================================================================
scrollUp:
if ($c.selected == 0); then goto mainLoop
set c.selected = $c.selected - 1
if ($c.selected < $menu.start);
	then set menu.start = $c.selected
call updateSelected
goto mainLoop

#================================================
scrollDown:
if ($c.selected == ($($(chooser).size) - 1)); then goto mainLoop
set c.selected = $c.selected + 1
if ($c.selected >= ($menu.start + $menu.height - 2))
	then set menu.start = $menu.start + 1
call updateSelected
goto mainLoop

#================================================
pageUp:
if ($($(chooser).size) < ($menu.height - 2));
	then goto pgupTop
if ($menu.start < ($menu.height - 3))
	then goto pgupTop
set menu.start = $menu.start - $menu.height + 3
set c.selected = $c.selected - $menu.height + 3
call updateSelected
goto mainLoop

pgupTop:
set menu.start = 0
set c.selected = 0
call updateSelected
goto mainLoop

#================================================
pageDown:
if ($($(chooser).size) < ($menu.height - 2));
	then goto pgdnSmall
if (($menu.start + 2 * ($menu.height - 2) - 1) > $($(chooser).size))
	then goto pgdnEnd
set menu.start = $menu.start + $menu.height - 3
set c.selected = $c.selected + $menu.height - 3
call updateSelected
goto mainLoop

pgdnEnd:
set menu.start = $($(chooser).size) - $menu.height + 2
set c.selected = $($(chooser).size) - 1
call updateSelected
goto mainLoop

pgdnSmall:
set c.selected = $($(chooser).size) - 1
call updateSelected
goto mainLoop

#================================================
escape:
if (!$c.allowEscape); then goto mainLoop
set c.escaped = 1
goto done

#================================================
finish:
set c.escaped = 0

done:
if ($c.mouse && !$_m.vis); then set mouseVisible = 0
if (defined[$(chooser).$(c.selected).value])
	then set c.value = "$($(chooser).$(c.selected).value)"
	else set c.value = "$($(chooser).$(c.selected).name)"
set c.finished = 1
set textCursorX = $_cursor.x
set textCursorY = $_cursor.y
set textColor = $_color
if (!$c.escaped && !compare["$($(chooser).$(c.selected).exec)" ""]); then $($(chooser).$(c.selected).exec)
if ($c.escaped && !compare["$c.onEscape" ""]); then $c.onEscape
script $c.onExit

#====================================================================
mouseHandler:
if ($mouseButtonLeft); then goto mouseClick
if (($mouseCursorX <= $menu.position.x) || ($mouseCursorX > $menu.extent.x - 2) || ($mouseCursorY <= $menu.position.y) || ($mouseCursorY >= $menu.extent.y))
	then goto mainLoop

# Hover effect.
set _m.sel = $menu.start - $menu.position.y + $mouseCursorY - 1
if ($_m.sel >= $($(chooser).size)); then goto mainLoop
set c.selected = $_m.sel
call updateSelected
goto mainLoop

# Click menu item or scroll bar
mouseClick:
if (($mouseCursorY <= $menu.position.y) || ($mouseCursorY >= $menu.extent.y))
	then goto mainLoop
if ($mouseCursorX == $menu.extent.x - 1)
	then goto mouseScroll
if (($mouseCursorX <= $menu.position.x) || ($mouseCursorX > $menu.extent.x - 3))
	then goto mainLoop

#================================================
# Click menu item
set _my = $mouseCursorY
set _m.sel = $menu.start + $mouseCursorY - $menu.position.y - 1
if ($_m.sel >= $($(chooser).size)); then goto mainLoop

set _mc = $c.menu.selected.color
set c.menu.selected.color = $c.menu.clicked.color
set c.selected = $_m.sel
call updateSelected
set c.menu.selected.color = $_mc

set string = " "
set rep.count = ($win.width - 3) - length[$($(list).$(_m.sel).name)] - 1
call repeatString

set _mc = 1	# Indicates highlighted state.
if (!$mouseButtonLeft); then goto finish

clickL:
mgetkey
if (!$mouseButtonLeft && $_mc); then goto finish
if (!$mouseButtonLeft && !$_mc); then goto mouseHandler
if ($lastKey != key[mouse]); then goto clickL
if ($mouseCursorY != $_my); then goto unclick
if (($mouseCursorX <= $menu.position.x) || ($mouseCursorX > $menu.extent.x - 3))
	then goto unclick

mHighlight:
if ($_mc); then goto clickL
set textCursorY = $_my
set textCursorX = $win.position.x + 1
set textColor = $c.menu.clicked.color
print " $($(chooser).$(_m.sel).name)$string.repeated"
set _mc = 1
goto clickL

unclick:
if (!$_mc); then goto clickL
set textCursorY = $_my
set textCursorX = $win.position.x + 1
set textColor = $c.menu.selected.color
print " $($(chooser).$(_m.sel).name)$string.repeated"
set _mc = 0
goto clickL

#================================================
# Scroll Bar
mouseScroll:
if ($mouseCursorY == $menu.position.y + 1)
	then goto scrollUp
if ($mouseCursorY == $menu.extent.y - 1)
	then goto scrollDown

set _m.sel = $mouseCursorY - $menu.position.y - 2
set _thumb = (($c.selected * 100) / ($($(chooser).size) - 1)) / (100 / ($win.height - 5))
if ($_m.sel < $_thumb); then goto pageUp
if ($_m.sel > $_thumb); then goto pageDown
set textColor = $scroll.color

set _l = ($($(chooser).size) - 1) / ($menu.height - 5)

thumbL:
mgetkey
if (!$mouseButtonLeft); then goto mainLoop
set _m.sel = $mouseCursorY - $menu.position.y - 2
if ($_m.sel == $_thumb); then goto thumbL
if (!$_thumb && ($_m.sel < $_thumb)); then goto thumbL
if (($_thumb == ($menu.height - 5)) && ($_m.sel > $_thumb))
	then goto thumbL

if ($_m.sel == ($menu.height - 5)); then goto thumbBot
if ($_l < ($menu.height - 5)); then goto lilWin

bigWin:
set menu.start = $_m.sel * $_l + (($_l - ($menu.height - 5)) / 2)
goto thumbUpdate

lilWin:
if ($($(chooser).size) < ($menu.height - 2));
	then goto thumbUpdate
set menu.start = $_m.sel * $_l - ((($menu.height - 5) - $_l) / 2)
if ($menu.start < 0); then set menu.start = 0
if ($menu.start > ($($(chooser).size) - $menu.height + 2))
	then set menu.start = $($(chooser).size) - $menu.height + 2

thumbUpdate:
set c.selected = ($_m.sel * (100 / ($menu.height - 5)) * ($($(chooser).size) - 1)) / 100
if ($_m.sel != (($c.selected * 100) / ($($(chooser).size) - 1)) / (100 / ($menu.height - 5)))
	then set c.selected = $c.selected + 1

call updateSelected
set _thumb = $_m.sel
goto thumbL

thumbBot:
if ($($(chooser).size) < ($menu.height - 2));
	then goto botSmall
set menu.start = $($(chooser).size) - $menu.height + 2
set c.selected = ($_m.sel * (100 / ($menu.height - 5)) * ($($(chooser).size) - 1)) / 100
if ($_m.sel != (($c.selected * 100) / ($($(chooser).size) - 1)) / (100 / ($menu.height - 5)))
	then set c.selected = $c.selected + 1
call updateSelected
set _thumb = $_m.sel
goto thumbL

botSmall:
set c.selected = $($(chooser).size) - 1
call updateSelected
set _thumb = $_m.sel
goto thumbL

#====================================================================


#================================================
#================================================
# SPECIAL-PURPOSE ROUTINES
#================================================
#================================================
updateSelected:
#set _color = $textColor
#set _cursor.x = $textCursorX
#set _cursor.y = $textCursorY

set textColor = $c.msg.text.color
set textCursorX = 6
set textCursorY = 17
print "$_msg.fill"
set textCursorX = 6
set textCursorY = 18
print "$_msg.fill"
set textCursorX = 6
set textCursorY = 19
print "$_msg.fill"
set textCursorX = 6
set textCursorY = 20
print "$_msg.fill"

set textCursorX = 6
set textCursorY = 17
if (defined[$(chooser).$(c.selected).msg.0])
	then print "$($(chooser).$(c.selected).msg.0)"
set textCursorX = 6
set textCursorY = 18
if (defined[$(chooser).$(c.selected).msg.1])
	then print "$($(chooser).$(c.selected).msg.1)"
set textCursorX = 6
set textCursorY = 19
if (defined[$(chooser).$(c.selected).msg.2])
	then print "$($(chooser).$(c.selected).msg.2)"
set textCursorX = 6
set textCursorY = 20
if (defined[$(chooser).$(c.selected).msg.3])
	then print "$($(chooser).$(c.selected).msg.3)"

#set textCursorX = $_cursor.x
#set textCursorY = $_cursor.y
#set textColor = $_color

#================================================
updateMenu:
set win.width = $menu.width
set win.height = $menu.height
set win.position.x = $menu.position.x
set win.position.y = $menu.position.y

set list = "$chooser"
set list.color = $c.menu.text.color
set list.selected.color = $c.menu.selected.color
set list.start = $menu.start
set list.selected = $c.selected
goto fillWindow

#================================================
#================================================
# GENERAL-PURPOSE ROUTINES
#================================================
#================================================
fillWindow:
# Fill a window with a list of items (requires window parameters).
#set _color = $textColor
#set _cursor.x = $textCursorX
#set _cursor.y = $textCursorY
set _y = $list.start
if (($($(list).size) - $list.start + 1) > ($win.height - 2))
	then set _max = $_y + $win.height - 2
	else set _max = $($(list).size)
set textColor = $list.color
set textCursorY = $win.position.y
set string = " "

fillL:
set textCursorX = $win.position.x + 1
set textCursorY = $textCursorY + 1

set rep.count = ($win.width - 3) - length[$($(list).$(_y).name)] - 1
call repeatString
if ($_y == $list.selected)
	then set textColor = $list.selected.color
	print " $($(list).$(_y).name)$string.repeated"
	then set textColor = $list.color
set _y = $_y + 1
if ($_y != $_max); then goto fillL

#set textCursorX = $_cursor.x
#set textCursorY = $_cursor.y
#set textColor = $_color

set scroll.percent = ($list.selected * 100) / ($($(list).size) - 1)
call drawScrollBar
return
#================================================

drawScrollBar:
# Draw a scroll bar (requires window parameters)
#	scroll.color - The color of the scroll bar.
#	scroll.percent - The position of the thumb (0 - 100).
#set _color = $textColor
#set _cursor.x = $textCursorX
#set _cursor.y = $textCursorY

set _scroll.x = $win.position.x + $win.width - 2
set textCursorX = $_scroll.x
set textCursorY = $win.position.y + 1
set textColor = $scroll.color
set _thumb = $scroll.percent / (100 / ($win.height - 5))
#if ($_thumb > ($win.height - 5)); then set _thumb = ($win.height - 5)
set _y = 0
print ""

_scrollL:
set textCursorX = $_scroll.x
set textCursorY = $textCursorY + 1
if ($_y == $_thumb); then print ""
else print ""
set _y = $_y + 1
if ($_y != ($win.height - 4)); then goto _scrollL

set textCursorX = $_scroll.x
set textCursorY = $textCursorY + 1
print ""
#set textCursorX = $_cursor.x
#set textCursorY = $_cursor.y
#set textColor = $_color
return
#================================================

drawWindow:
# Draw a plain window.
#	win.color - The color of the window and its border.
#	win.shadow.color - The color of the shadow.
#	win.width - The width of the window (not counting shadow).
#	win.height - The height of the window (not counting shadow).
#	win.position.x - The X position of the window.
#	win.position.y - The Y position of the window.
#set _color = $textColor
#set _cursor.x = $textCursorX
#set _cursor.y = $textCursorY

# Make the window background
set rep.count = $win.width - 2
set string = ""
call repeatString
set _win.top = "$string.repeated"
set string = " "
call repeatString
set _win.row = "$string.repeated"
set string = ""
call repeatString
set _win.bottom = "$string.repeated"
set string = ""
call repeatString
set _win.shadow = "$string.repeated"

# Draw the window background
# top:
set textColor = $win.color
set textCursorX = $win.position.x
set textCursorY = $win.position.y
print "$_win.top"
set textColor = $win.shadow.color
print ""
set _y = 0

# middle:
_winLoop:
set textColor = $win.color
set textCursorX = $win.position.x
set textCursorY = $textCursorY + 1
print "$_win.row"
set textColor = $win.shadow.color
print ""
set _y = $_y + 1
if ($_y != $win.height - 2); then goto _winLoop

# bottom:
set textColor = $win.color
set textCursorX = $win.position.x
set textCursorY = $textCursorY + 1
print "$_win.bottom"
set textColor = $win.shadow.color
print ""
set textCursorX = $win.position.x + 1
set textCursorY = $textCursorY + 1
print "$_win.shadow"

#set textCursorX = $_cursor.x
#set textCursorY = $_cursor.y
#set textColor = $_color
return
#================================================

# Plain background draw routine.
# Call with bg.char set to whatever character you want to
# paint with, and bg.color set to whatever color you want.
drawBackground:
#set _color = $textColor
#set _cursor.x = $textCursorX
#set _cursor.y = $textCursorY
set textColor = $bg.color
set textCursorX = 0
set textCursorY = $bg.start
set string = "$bg.char"
set rep.count = 80
call repeatString
set _y = 0

_bgLoop:
set _y = $_y + 1
print "$string.repeated"
if ($_y != $bg.size); then goto _bgLoop

#set textCursorX = $_cursor.x
#set textCursorY = $_cursor.y
#set textColor = $_color
return
#================================================

# String duplicator.
# Call with $string set to the string to duplicate,
# and $rep.count set to the requested repeat count (<128).
# Returns $string.repeated
repeatString:
set string.repeated = ""
set _r2 = "$(string)$string"
set _r4 = "$(_r2)$_r2"
set _r8 = "$(_r4)$_r4"
set _r16 = "$(_r8)$_r8"
set _r32 = "$(_r16)$_r16"
set _r64 = "$(_r32)$_r32"
if ($rep.count & 64); then set string.repeated = "$_r64"
if ($rep.count & 32); then set string.repeated = "$(string.repeated)$_r32"
if ($rep.count & 16); then set string.repeated = "$(string.repeated)$_r16"
if ($rep.count & 8); then set string.repeated = "$(string.repeated)$_r8"
if ($rep.count & 4); then set string.repeated = "$(string.repeated)$_r4"
if ($rep.count & 2); then set string.repeated = "$(string.repeated)$_r2"
if ($rep.count & 1); then set string.repeated = "$(string.repeated)$string"
return;
#================================================
